'  Copyright(C)2009 Rayslogic.com, LLC
'Show a video (now with audio) on TV!
'video data created with special software and stored on SD card
'
'  Brought to you by:  Rayslogic.com
'  The Propeller System Module is here!  Now available at Gadget Gangster
'
'  'Bitmap buffer fills most of memory...
'  'a few tricks are used to conserve space

'  Pin settings are for PSM with PAB...
'  If you're using a Demo or Proto board, you'll need to change pin settings.
'  Read the comments in the CON section below for more info

CON

  _clkmode = xtal1 + pll16x
  _xinfreq =5_000_000
  _STACK=40      'going to reserve a little stack space, otherwise things can go very wrong!
  x_tiles = 16
  y_tiles = 12

  SD_DO=3              '********   SPECIFY SD Card Pins HERE   *****************
  SD_CLK=2             'These settings are for the PSM
  SD_DI=1              'But, often DO is 0 and CS is 3 (i.e, reversed order)...
  SD_CS=0   

  
  AudioFrameSize = 735
  AudioSampleRate=22050
  AudioLeftPin = 4         '********   SPECIFY Audio Pins HERE   *****************    
  AudioRightPin = 5
  AudioAttenuation = 0  '  <----  increase this number a little to reduce volume, if desired

  'Note:  Demo/Proto board users will need to change TV pins definitions!!!!!!!!!!!!!!
  '       Look far below for this Dat section:  "DAT       'TV Settings" to find where to change TV pin settings   

  
OBJ

  tv    : "TV_PsmVidMod"     'Show pictures on TV!
  sd  : "fsrw"   'Controls the SD card  
    
DAT  'The Bitmap           

sFilename byte
byte "ProfProp.dat",0        '********   SPECIFY Video Filename HERE   *****************



'**********************************************************************************

VAR

  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only

  word  screen[x_tiles * y_tiles]
  long  colors[64] 

  long  row,col,flag,screensize,rows,cols,color

var
  word audioBuffer1[AudioFrameSize]
  word audioBuffer2[AudioFrameSize]
  long parameter1  'to pass @buff1 to ASM
  long parameter2  'to pass @buff2 to ASM
  long parameter3  'to pass sample rate to ASM
  long parameter4  'to pass #samples to ASM


PUB start | i, pBmp, picBytes, acnt, nframes, dcnt
  'Start up the modified TV driver and show some text
  'start tv
  longmove(@tv_status, @tvparams, 14)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)   

  rows:=y_tiles
  cols:=x_tiles
  screensize:=rows*cols
  'setcolors(@palette)       'this code commented out to save space, adding the following 2 lines instead...
  colors[0]:=$070A070A
  colors[1]:=$07070a0a
  out(0)
  str(string("TV Video Viewer"))


  
  'specify parameters
  biWidth:=160
  biHeight:=120 
  picBytes:=(4*((biWidth+3)/4))*biHeight
  'Prepare then show the bitmap
  pBmp:=@Bitmap+160*120'DeIndexColors(@Bitmap)
  'the last color value (63) is used to pass bitmap info to the driver
  colors[63]:=(4*((biWidth+3)/4))<<24+biHeight<<16+pBmp   'pass pointer to BMP

   'You can turn off the bitmap with the following line
'  colors[63]:=0

  out(0)
  str(string("Mounting...") )
  
  if (sd.mount_Explicit(SD_DO,SD_CLK,SD_DI,SD_CS)<0)
  'if (sd.mount<0) 
    out(0)  
    str(string("SD Fail!"))
  else
    out(0)
    str(string("SD Mounted."))
 
  repeat 1 
    waitcnt(cnt+clkfreq)
    out(0)
    str(string("Opening Video"))
    if (sd.popen(@sFilename,"r")<0)
      out(0)  
      str(string("Open Failed"))
      quit
    else
      out(0)
      str(string("File Open"))
     
    out(0)
    str(string("Playing"))
     
    i:=1



    'calculate number of video frames in file
    nframes:=sd.Get_FileSize/(160*120+AudioFrameSize<<1)       '20670 

    parameter1:=@audiobuffer1
    parameter2:=@audiobuffer2
    parameter3:=CLKFREQ/AudioSampleRate  
    parameter4:=nframes*AudioFrameSize
    dcnt:=clkfreq/35 

    'preload first frame of audio
    sd.pread(@audioBuffer1,constant(AudioFrameSize<<1))


    'Start ASM audio player in a new cog
    COGNEW(@ASMWAV,@parameter1)

    sd.pread(@Bitmap,picBytes)    'load first video frame
    nframes--

    acnt:=cnt    
    
    repeat nframes>>1
      sd.pread(@audioBuffer2,constant(AudioFrameSize<<1))
      sd.pread(@Bitmap,picBytes)
      repeat until word[@audiobuffer2]==0

      
      sd.pread(@audioBuffer1,constant(AudioFrameSize<<1))
      sd.pread(@Bitmap,picBytes) 
      repeat until word[@audiobuffer1]==0 

      

    sd.pclose
    row:=0
    col:=0
    str(string("Done.   "))  


   
    
PUB out(c) | i, k

'' Output a character
''
''     $00 = clear screen
''     $01 = home
''     $08 = backspace
''     $09 = tab (8 spaces per)
''     $0A = set X position (X follows)
''     $0B = set Y position (Y follows)
''     $0C = set color (color follows)
''     $0D = return
''  others = printable characters

  case flag
    $00: case c
           $00: wordfill(@screen, $220, screensize)
                col := row := 0
           $01: col := row := 0
           $08: if col
                  col--
           $09: repeat
                  print(" ")
                while col & 7
           $0A..$0C: flag := c
                     return
           $0D: newline
           other: print(c)
    $0A: col := c // cols
    $0B: row := c // rows
    $0C: color := c & 7
  flag := 0

  
PRI print(c)|k,i
      k := color << 1 + c & 1
      i := k << 10 + $200 + c & $FE
      screen[row * cols + col] := i
      screen[(row + 1) * cols + col] := i | 1
      if ++col == cols
        newline


PRI newline : i

  col := 0
  if (row += 2) == rows
    row -= 2
    'scroll lines
    repeat i from 0 to rows-3
      wordmove(@screen[i*cols], @screen[(i+2)*cols], cols)
    'clear new line
    wordfill(@screen[(rows-2)*cols], $220, cols<<1)


PUB str(stringptr)

'' Print a zero-terminated string

  repeat strsize(stringptr)
    out(byte[stringptr++])


PUB dec(value) | i

'' Print a decimal number

  if value < 0
    -value
    out("-")

  i := 1_000_000_000

  repeat 10
    if value => i
      out(value / i + "0")
      value //= i
      result~~
    elseif result or i == 1
      out("0")
    i /= 10



DAT       'TV Settings

tvparams                long    0               'status
                        long    1               'enable
                        long    %010_0101       'pins for Propeller System Module
                        'long    %001_0101       'pins for Demo Board
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    0               'broadcast
                        long    0               'auralcog



DAT     'Data for reading bitmap files

BMPHeader  byte 'Mostly using info from here:  http://www.fortunecity.com/skyscraper/windows/364/bmpffrmt.html
bfType byte "B","M"  
bfSize long 0
bfReserved1 word 0
bfReserved2 word 0
bfOffBits long 54
biSize long 40
biWidth long 0
biHeight long 10
biPlanes word 1
biBitCount word 24
biCompression long 0
biSizeImage long 0
biXPelsPerMeter long 0
biYPelsPerMeter long 0
biClrUsed long 0
biClrImportant long 0


BmpPalette long 0[256]      'container for bmp palette entries

{
DAT  'text palette
                        '       fore   back
                        '       color  color
palette                 byte    $07,   $0A    '0    white / dark blue
                        byte    $07,   $BB    '1    white / red
                        byte    $9E,   $9B    '2   yellow / brown
                        byte    $04,   $07    '3     grey / white
                        byte    $3D,   $3B    '4     cyan / dark cyan
                        byte    $6B,   $6E    '5    green / gray-green
                        byte    $BB,   $CE    '6      red / pink
                        byte    $3C,   $0A    '7     cyan / blue
}

DAT    'Palette

TV_Palette long
byte    $02,$03,$04,$05,$06,$07
byte    $08,$0A,$0B,$0c,$0d,$0e
byte    $18,$1A,$1B,$1C,$1D,$1E
byte    $28,$2A,$2B,$2C,$2D,$2E
byte    $38,$3A,$3B,$3C,$3D,$3E
byte    $48,$4A,$4B,$4C,$4D,$4E
byte    $58,$5A,$5B,$5C,$5D,$5E
byte    $68,$6A,$6B,$6C,$6D,$6E
byte    $78,$7A,$7B,$7C,$7D,$7E
byte    $88,$8A,$8B,$8C,$8D,$8E
byte    $98,$9A,$9B,$9C,$9D,$9E
byte    $A8,$AA,$AB,$AC,$AD,$AE
byte    $B8,$BA,$BB,$BC,$BD,$BE
byte    $C8,$CA,$CB,$CC,$CD,$CE
byte    $D8,$DA,$DB,$DC,$DD,$DE
byte    $E8,$EA,$EB,$EC,$ED,$EE
byte    $F8,$FA,$FB,$FC,$FD,$FE


DAT
TvPalette long
'file "TvPalette.pal"

DAT

Bitmap long
long    $02030405[160*120/4-304/4]  'subtract off size of assembly wav player

DAT
  ORG 0
ASMWAV
'load input parameters from hub to cog given address in par
        movd    :par,#pData1             
        mov     x,par
        mov     y,#4  'input 4 parameters
:par    rdlong  0,x
        add     :par,dlsb
        add     x,#4
        djnz    y,#:par

setup
        'setup output pins
        MOV DMaskR,#1
        ROL DMaskR,OPinR
        OR DIRA, DMaskR
        MOV DMaskL,#1
        ROL DMaskL,OPinL
        OR DIRA, DMaskL
        'setup counters
        OR CountModeR,OPinR
        MOV CTRA,CountModeR
        OR CountModeL,OPinL
        MOV CTRB,CountModeL
        'Wait for SPIN to fill table
        MOV WaitCount, CNT
        ADD WaitCount,BigWait
        WAITCNT WaitCount,#0
        'setup loop table
        MOV LoopCount,SizeBuff  
        'ROR LoopCount,#1    'for stereo
        MOV pData,pData1
        MOV nTable,#1
        'setup loop counter
        MOV WaitCount, CNT
        ADD WaitCount,dRate


MainLoop
        SUB nSamples,#1
        CMP nSamples,#0 wz
        IF_Z JMP #Done
        waitcnt WaitCount,dRate

        RDLONG Right,pData
        ROL Right,#16       'mono word data
        MOV   Left,Right
        SAR   Right,#AudioAttenuation
        
        SAR   Left,#AudioAttenuation
        ADD Right,twos      'Going to cheat a bit with the LSBs here...  Probably shoud fix this!
        Add Left,twos
        MOV FRQA,Right
        
        MOV FRQB,Left'Right
        WRLONG Zero,pData
        ADD pData,#2     'Mono word data

        'loop
        DJNZ LoopCount,#MainLoop
        
        MOV LoopCount,SizeBuff        
        'switch table       ?
        CMP nTable,#1 wz
        IF_Z JMP #SwitchToTable2
SwitchToTable1
        MOV nTable,#1
        MOV pData,pData1
        JMP #MainLoop
SwitchToTable2
        MOV nTable,#2
        MOV pData,pData2
        JMP #MainLoop
        
                
Done
         'now stop
        COGID thisCog
        COGSTOP thisCog          

'Working variables
thisCog long 0
x       long 0
y       long 0
dlsb    long    1 << 9
BigWait long 100000
twos    long $8000_8000
        
'Loop parameters
nTable  long 0
WaitCount long 0
pData   long 0
LoopCount long 0
SizeBuff long AudioFrameSize
'Left    long 0
Right   long 0
Left    long 0
Zero    long 0          

'setup parameters
DMaskR  long 0 'right output mask
OPinR   long AudioRightPin 'right channel output pin #              
DMaskL  long 0 'left output mask 
OPinL   long AudioLeftPin 'left channel output pin #                 
CountModeR long %00011000_00000000_00000000_00000000
CountModeL long %00011000_00000000_00000000_00000000


'input parameters
pData1   long 0 'Address of first data table        
pData2   long 0 'Address of second data table
dRate    long 5000  'clocks between samples
nSamples long 2000